/* FILE: sine.dsp               Copyright (c), Symmetric Research, 1992-1999
 *
 * This program computes the function sin(x), where x is in radians and
 * in the range [-PI/2,PI/2].
 *
 * The sine is computed by polynomial approximation, the algorithm being
 * the same as the one listed in the AT&T/Lucent Application Software
 * Library Reference manual.  They took their optimized polynomial from
 * Cody and Waite, 1980.
 *
 * Many of DSPASM's language features are not used here.  This example
 * shows how to directly transcribe typical AT&T/Lucent source code.
 *
 *
 * The form of the polynomial is:
 *
 *      sin(x) = x + c1*x^3 + c2*x^5 + c3*x^7 + c4*x^9
 *
 * and is computed in the program as
 *
 *      sin(x) = x + x^3*(c1 + c2*x^2) + x^4*x^3*(c3 + c4*x^2)
 *
 * where subtle use is made of the latency in DAU instructions to save
 * clocks.  Follow the use of accumulator a3 carefully as an example.
 *
 * To run this program, assemble it with DSPASM, and then download and
 * execute it by hand in the DSPMON monitor.  Check the value of the
 * result before and after the program is run.
 *
 *
 *
 */

constant

        PROGRAM         = 0x000000,
        DATA            = 0x000100;




<DATA>

dspf

    x = 1.0, result = 0.0; /* result = sin( x), poke whatever you like in x */


dspf c[] = { /* polynomial coefficients c4, c3, c2, c1 */

        0.2601903036E-5, -0.198074182E-3,
        0.8333025139E-2, -0.1666665668

        };





<PROGRAM>

sin {

        /* Point to x and the coefficient array. */

        r1 = x;
        r2 = c;


        /* Compute the polynomial approximation. */

        a2 = *r1 * *r1;                 /* a2 = x^2             */
        a0 = *r1;                       /* a0 = x               */
        nop;
        a3 = a2 * *r1;                  /* a3 = x^3             */


        a2 = *r2++ + a2 * *r2++;        /* c3 + c4*x^2          */
        a1 = *r2++ + a2 * *r2++;        /* c1 + c2*x^2          */

        a3 = a3 * a0;                   /* a3 = x^4             */
        a2 = a2 * a3;                   /* c3*x^3 + c4*x^5      */

        a1 = a0 + a1 * a3;              /* x + c1*x^3 + c2*x^5  */

        nop;
        a0 = a1 + a2 * a3;      /* x + c1*x^3 + c2*x^5 + c3*x^7 + c4*x^9 */



        /* Save the result. */

        r1 = result; *r1 = a0 = round( a0);


        /* Wait for the PC. */

        wait: goto wait;

}
